import React, {useState} from 'react';
import './App.css';
import MultipleSelectFilter from './MultipleSelectFilter.tsx';
import MultipleSelectModule from './MultipleSelectModule.tsx';

import { Grid, TextField, List, ListItem, Divider, Card, Typography} from '@mui/material';
import { Timeline, TimelineItem, TimelineSeparator, TimelineConnector, TimelineContent, TimelineDot} from '@mui/lab';
import TimelineOppositeContent, {
  timelineOppositeContentClasses,
} from '@mui/lab/TimelineOppositeContent';

import DonutLargeIcon from '@mui/icons-material/DonutLarge';
import SquareFootIcon from '@mui/icons-material/SquareFoot';
import StraightenIcon from '@mui/icons-material/Straighten';
import AccountBalanceIcon from '@mui/icons-material/AccountBalance';
import WarehouseIcon from '@mui/icons-material/Warehouse';
import EuroIcon from '@mui/icons-material/Euro';
import PowerIcon from '@mui/icons-material/Power';
import LocalFireDepartmentIcon from '@mui/icons-material/LocalFireDepartment';
import ConstructionIcon from '@mui/icons-material/Construction';
import HeightIcon from '@mui/icons-material/Height';

import BuildIcon from '@mui/icons-material/Build';
import DynamicFeedIcon from '@mui/icons-material/DynamicFeed';
import HubIcon from '@mui/icons-material/Hub';
import InfoIcon from '@mui/icons-material/Info';

import FileCopyIcon from '@mui/icons-material/FileCopy';
import IconButton from '@mui/material/IconButton';

import { ThemeProvider, createTheme } from '@mui/material/styles';



let theme = createTheme({
  palette: {
    primary: {
      main: '#049a9b',
    }
  },
});


const listItemStyle = { margin: '-15px 5px -10px 0px',};
const infoTitlesStyle = { fontWeight: 'bold',  margin: '15px 0px 10px 8px',  fontSize: '25px' };
const informacaoTitleStyles = {fontWeight: 'bold', margin: '0px 10px 5px 10px',  fontSize: '15px' }
const informacaoContentStyle = { margin: '0px 10px 5px 10px', fontSize: '15px' }

const namesWithIcons = [
  { id: 0, name: 'Novas funcionalidade', icon: < HubIcon /> },
  { id: 1, name: 'Correção de Falhas', icon: <BuildIcon /> },
  { id: 2, name: 'Melhorias', icon: <DynamicFeedIcon /> },
];


function formatDate(myString){
  const date = new Date(myString);
  const year = date.getUTCFullYear() % 100; // Get the last two digits of the year
  const month = (date.getUTCMonth() + 1).toString().padStart(2, '0'); // Month is zero-based
  const day = date.getUTCDate().toString().padStart(2, '0');
  
  return `${day}/${month}/${year}`;
}


function semanticVersions(myString){
  // Convert from "005001010" to 5.1.10
  return String(parseInt(myString.slice(0, 3)))+"."+ 
         String(+ parseInt(myString.slice(3, 6)))+"."+ 
         String(+ parseInt(myString.slice(6, 9)))

}

const getIconForModule = (myModuleName) => {
  switch (myModuleName) {
    case 'CORE':  return <DonutLargeIcon />;
    case 'FOR':   return <SquareFootIcon />;
    case 'IE':    return <PowerIcon/>;
    case 'LME':
    case 'LAB':   return <StraightenIcon/>;
    case 'ELV':   return <HeightIcon />;
    case 'FIN':   return <AccountBalanceIcon />;
    case 'ATV':   return <AccountBalanceIcon />;
    case 'PAD':   return <EuroIcon />;
    case 'OBL':
    case 'OBLER': return <WarehouseIcon/>;
    case 'GAS':   return <LocalFireDepartmentIcon/>;
    case 'FUN':   return <ConstructionIcon />;
    default:      return <AccountBalanceIcon />;
  }
};


function VersionComponent({message, setSelectedVersion, filters }) {

  //Check for the tab selected --> Could be "SGI" or "Mobile"
  var tabSelected = "";
  if (message.name === "SGI Mobile") {tabSelected = "mobile"}
  else {tabSelected = "SGI"}

  // Versions (TIMELINE)
  let versions = message.versions   //Get all versions
  const [selectedVersion, setSelectedIcon] = useState(null);    


  // Module (CHIP)   
  const moduleNames: string[] = [];
  for (let i = 0; i < message.modules.length; i++){ moduleNames[i] = message.modules[i].value;}

  // Filters
  const [moduleFilter, setModuleFilter] = useState([]);
  const [filter, setFilter] = useState([]);


  // Handling of the version click
  const handleIconClick = (index, data) => {
    setSelectedIcon(index); // Aggiorna lo stato selectedVersion
    setSelectedVersion(data);
  };

  // Handle search text
  const [searchText, setSearchText] = useState('');

   // Function for filtering the versions by the search text
   const cleanedVersions = Object.values(versions).filter((item) => {
    const versionInfo = `${item.module.value} ${semanticVersions(item.version)} ${item.improvements} ${item.bugfixes} ${item.added}`;
    return versionInfo.toLowerCase().includes(searchText.toLowerCase());
  });

  // -------------------------------------------------------------------
  // -------------------------------------------------------------------
  // -------------------------------------------------------------------
  


    return (
      <ThemeProvider theme={theme}>
        <Card className="Versoes-Card" variant="outlined">
          <Typography sx={{ fontWeight: 'bold', margin: '15px 15px 10px 15px', fontSize: '25px' }}>
              Versões {tabSelected}
          </Typography>

          <TextField 
              size="small" 
              id="outlined" 
              label="Pesquisa..."
              value={searchText}
              onChange={(e) => setSearchText(e.target.value)}
              sx={{ width: 'calc(100% - 30px)', marginLeft: '15px', marginRight: '15px', marginBottom: '15px'}}
              />
   

        <Grid container sx={{ marginBottom: '10px'}}>

          <Grid item sx={{marginLeft: "8px", marginTop: "-5px"}}>
            <MultipleSelectModule modules={message.modules} moduleFilter={moduleFilter} setModuleFilter={setModuleFilter}></MultipleSelectModule>
          </Grid>

          <Grid item sx={{marginLeft: "8px", marginTop: "-5px", paddingRight: '5px'}}>
            <MultipleSelectFilter names={namesWithIcons} filter={filter} setFilter={setFilter}></MultipleSelectFilter>
          </Grid>         

        </Grid>


          <Timeline style={{maxHeight: '800px', overflow: 'auto'}} 
            sx={{[`& .${timelineOppositeContentClasses.root}`]: {flex: 0.1, },}}> 
            {cleanedVersions.map((item, index) => {

              // Filter item by the modulew selected (module chip)
              if (moduleFilter.length > 0){
                if (!(moduleFilter.includes(item.module.value))){return null;} 
              }

              //console.log("my item", item.module.value)

              // Filter module by the improvement chip
              if ((filter.includes(namesWithIcons[2].name)) && ((item.improvements === undefined)  || (item.improvements === "")) ) {return null;}
              // Filter module by the added chip
              if ((filter.includes(namesWithIcons[0].name)) && ((item.added === undefined)  || (item.added === ""))) {return null;}
              // Filter module by the added chip
              if ((filter.includes(namesWithIcons[1].name)) && ((item.bugfixes === undefined) || (item.bugfixes === ""))) {return null;}
              

              // Return the item 
              return (
                <TimelineItem key={index} sx={{ py: '4px' }}>

                  <TimelineOppositeContent color="textSecondary" sx={{ py: '22px' }}>
                    <Typography variant="body2" component="div">
                      {formatDate(item.date)}
                    </Typography>
                    
                  </TimelineOppositeContent>


                  <TimelineSeparator>

                    <TimelineDot
                      color={selectedVersion === index ? 'warning' : 'primary'}
                      onClick={() => handleIconClick(index, item)}> 
                      {getIconForModule(item.module.name)}
                    </TimelineDot>

                    <TimelineConnector sx={{ paddingTop: '22px' }} />

                  </TimelineSeparator>


                  <TimelineContent>
                    <Typography variant="h6" sx={{ fontSize: '18px', lineHeight: '18px', padding: '8px 0px 0px 0px' }}>
                      {item.module.value}
                    </Typography>
                    <Typography variant="body2" color="textPrimary">
                      {semanticVersions(item.version)}
                    </Typography>
                  </TimelineContent>

                </TimelineItem>
              );
            })}

          </Timeline>

          {console.log("NO VERSIOooN!", cleanedVersions.length)}
          {console.log("\n\n\n\n\n\n\nYALLE!")}


        </Card>
      </ThemeProvider>

    );
  }

function InfoComponent({ modules, version: selectedVersion }) {
  
    // If no version is selected, don't render anything
    if (!selectedVersion || !selectedVersion.version) {
      return null; 
    }

  
  const copyText = (text) => {
    // Function that creates an icon to copy the git id
    const tempInput = document.createElement('input');
    tempInput.value = text;
    document.body.appendChild(tempInput);

    tempInput.select();
    tempInput.setSelectionRange(0, 999);

    document.execCommand('copy');

    document.body.removeChild(tempInput);
    console.log(`Copied: ${text}`);
  };

  return (
    <ThemeProvider theme={theme}>
    
    <Card variant="outlined">
      <List >
          <ListItem sx={listItemStyle}> 
          <InfoIcon></InfoIcon>
          <Typography sx={infoTitlesStyle}>
            Informação para versão {semanticVersions(selectedVersion.version)}
          </Typography> 

          </ListItem>

          <ListItem>
            <Grid container spacing={3}>
              <Grid item md={6}>
                <Typography sx={informacaoTitleStyles}> Módulo </Typography>
                <Typography sx={informacaoContentStyle}>
                  {selectedVersion.module.value}
                </Typography>
              </Grid>

              <Grid item md={6}>
                <Typography sx={informacaoTitleStyles}> Data de lançamento </Typography>
                <Typography sx={informacaoContentStyle}>
                {formatDate(selectedVersion.date)}
                </Typography>
              </Grid>

              <Grid item md={12}>
               <Typography sx={informacaoTitleStyles}> Revisão GIT (se aplicavel) </Typography>
                <Typography sx={informacaoContentStyle}>
                  {selectedVersion.gitHash ? 
                  (
                    <>
                      {selectedVersion.gitHash}
                      <IconButton sx={{ transform: 'scale(0.6)', margin: '-10px 0px 0px 0px' }}
                                  title="Copia" 
                                  onClick={() => copyText(selectedVersion.gitHash)}
                      >
                        <FileCopyIcon />
                      </IconButton>
                    </>
                  ) : (
                    "N/D"
                  )}

                </Typography>

              </Grid>
            </Grid>
          </ListItem>

      </List>

      <Divider/>

      {selectedVersion.added && (
          <List>
              <ListItem sx={listItemStyle}> 
              {namesWithIcons[0].icon}
              <Typography sx={infoTitlesStyle}>

               Novas Funcionalidades
              </Typography>

              </ListItem>
              <ListItem>
              <Typography sx={{ margin: '0px 0px 10px -10px', fontSize: '15px' }}
               dangerouslySetInnerHTML={{ __html: selectedVersion.added }} />
              </ListItem>
          </List>
      )}

      <Divider/>

      {selectedVersion.bugfixes && (
          <List>
              <ListItem sx={listItemStyle}> 
              {namesWithIcons[1].icon}
              <Typography sx={infoTitlesStyle}>
                Correção de Falhas
              </Typography>
              </ListItem>
              <ListItem>
              <Typography sx={{ margin: '0px 0px 10px -10px', fontSize: '15px' }}
                 dangerouslySetInnerHTML={{ __html: selectedVersion.bugfixes }} />
  
              </ListItem>

          </List>
      )}

    <Divider/>

    {selectedVersion.improvements && (
          <List>
              <ListItem sx={listItemStyle}> 
              {namesWithIcons[2].icon}
              <Typography sx={infoTitlesStyle}>
                Melhorias
              </Typography> 
              
              </ListItem>
              <ListItem>
                  <Typography sx={{ margin: '0px 0px 10px -10px', fontSize: '15px' }}
                    dangerouslySetInnerHTML={{ __html: selectedVersion.improvements }} />

              </ListItem>

          </List>
          
      )}

      <Divider/>



    </Card>

    </ThemeProvider>
  );
}


export {VersionComponent, InfoComponent};