import * as React from 'react';
import Box from '@mui/material/Box';
import OutlinedInput from '@mui/material/OutlinedInput';
import InputLabel from '@mui/material/InputLabel';
import MenuItem from '@mui/material/MenuItem';
import FormControl from '@mui/material/FormControl';
import Select, { SelectChangeEvent } from '@mui/material/Select';
import Chip from '@mui/material/Chip';
import Grid from '@mui/material/Grid';


interface NameWithIcon {
  name: string;
  icon: JSX.Element; // Tipo di elemento per l'icona
}

interface Props {
  names: NameWithIcon[]; // Modifica il tipo delle props per accettare nomi con icone
  filter: string[]; // Aggiungi il tipo di props per l'array filter

}

const MultipleSelectFilter: React.FC<Props> = ({ names, filter, setFilter }) => {
  const [personName, setPersonName] = React.useState<string[]>([]);

  const handleChange = (event: SelectChangeEvent<typeof personName>) => {
    const {
      target: { value },
    } = event;
    const selectedValues = typeof value === 'string' ? value.split(',') : value;
    setPersonName(selectedValues);
    setFilter(selectedValues); // Aggiorna l'array filter quando cambia la selezione
  };

  return (
    <Grid>
      <FormControl sx={{ m: 1, minWidth: 240}} >
        <InputLabel id="demo-multiple-chip-label" sx={{ marginTop: '-8px' }}>Filtra </InputLabel>
        <Select
          labelId="demo-multiple-chip-label"
          //id="demo-multiple-chip"
          multiple
          size="small" 
          value={personName}
          onChange={handleChange}
          input={<OutlinedInput id="select-multiple-chip" label="Chip" />}
          renderValue={(selected) => (
            <Box sx={{ display: 'flex', flexWrap: 'wrap', gap: '0.8px' }}>
              {selected.map((value) => (
                <Chip 
                  key={value} 
                  label={names.find((item) => item.name === value)?.icon}
                  sx={{ fontSize: '2rem'}}/> 
              ))}
            </Box>
          )}
        >

          {names.map((nameWithIcon) => (
            <MenuItem key={nameWithIcon.name} value={nameWithIcon.name}>
              <Box sx={{ display: 'flex', alignItems: 'center' }}>
                {nameWithIcon.icon}
                <span style={{ marginLeft: '10px' }}>{nameWithIcon.name}</span>
              </Box>
            </MenuItem>
          ))}
        </Select>
      </FormControl>
    </Grid>
  );
}

export default MultipleSelectFilter;
