import * as React from 'react';

import Box from '@mui/material/Box';
import OutlinedInput from '@mui/material/OutlinedInput';
import InputLabel from '@mui/material/InputLabel';
import MenuItem from '@mui/material/MenuItem';
import FormControl from '@mui/material/FormControl';
import Select, { SelectChangeEvent } from '@mui/material/Select';
import Chip from '@mui/material/Chip';
import Grid from '@mui/material/Grid';

import DonutLargeIcon from '@mui/icons-material/DonutLarge';
import SquareFootIcon from '@mui/icons-material/SquareFoot';
import StraightenIcon from '@mui/icons-material/Straighten';
import AccountBalanceIcon from '@mui/icons-material/AccountBalance';
import WarehouseIcon from '@mui/icons-material/Warehouse';
import EuroIcon from '@mui/icons-material/Euro';
import PowerIcon from '@mui/icons-material/Power';
import LocalFireDepartmentIcon from '@mui/icons-material/LocalFireDepartment';
import ConstructionIcon from '@mui/icons-material/Construction';
import HeightIcon from '@mui/icons-material/Height';

const getIconForModule = (moduleName) => {
  switch (moduleName) {
    case 'CORE':  return <DonutLargeIcon />;
    case 'FOR':   return <SquareFootIcon />;
    case 'IE':    return <PowerIcon/>;
    case 'LME':
    case 'LAB':   return <StraightenIcon/>;
    case 'ELV':   return <HeightIcon />;
    case 'FIN':   return <AccountBalanceIcon />;
    case 'ATV':   return <AccountBalanceIcon />;
    case 'PAD':   return <EuroIcon />;
    case 'OBL':
    case 'OBLER': return <WarehouseIcon/>;
    case 'GAS':   return <LocalFireDepartmentIcon/>;
    case 'FUN':   return <ConstructionIcon />;
    default:      return <AccountBalanceIcon />;
  }
};

interface Props {
  modules: { value: string }[];
  moduleFilter: string[]; // Aggiungi il tipo di props per l'array filter
  setModuleFilter: (filter: string[]) => void;
}

const MultipleSelectModule: React.FC<Props> = ({ modules, moduleFilter, setModuleFilter }) => {
  const [selectedModules, setSelectedModules] = React.useState<string[]>([]);

  const handleChange = (event: SelectChangeEvent<typeof selectedModules>) => {
    console.log("QUi--> ", event.target, event.target.value, event.target.name)
    setSelectedModules(event.target.value as string[]);
    
    const selectedValues = (event.target.value as { value: string, name: string }[]).map(module => module.value);
    setModuleFilter(selectedValues); // Update moduleFilter with only the 'value' properties
  };

  return (
    <Grid>
  <FormControl sx={{ m: 1, minWidth: 240}}>
    <InputLabel id="demo-multiple-chip-label" sx={{ marginTop: '-8px' }}>Selecionar modulo</InputLabel>
    <Select
      fullWidth
      labelId="demo-multiple-chip-label"
      //id="demo-multiple-chip"
      multiple
      size="small"
      value={selectedModules}
      onChange={handleChange}
      input={<OutlinedInput id="select-multiple-chip" label="Chip" />}
      renderValue={(selected) => {
        return (
          <Box sx={{ display: 'flex', flexWrap: 'wrap', gap: '0.1px' }}>
            {selected.map((value) => (
              <Chip 
                key={value} 
                icon={getIconForModule(value.name)}
                sx={{
                  fontSize: '2rem', backgroundColor: 'transparent'}} />
            ))}
          </Box>
        );
      }}
    >
      {modules.map((modulee) => (
        <MenuItem key={modulee} value={modulee}>
          <Box sx={{ display: 'flex', alignItems: 'right' }}>
            {getIconForModule(modulee.name)}
            <span style={{ marginLeft: '10px' }}>{modulee.value}</span>
          </Box>
        </MenuItem>
      ))}
    </Select>
  </FormControl>
</Grid>

  );
}

export default MultipleSelectModule;
